/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3) 
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details. 
 */

package org.dive4elements.river.importer;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import org.hibernate.Session;
import org.hibernate.Query;

import org.dive4elements.river.model.GrainFraction;
import org.dive4elements.river.model.River;
import org.dive4elements.river.model.SedimentYield;
import org.dive4elements.river.model.TimeInterval;
import org.dive4elements.river.model.Unit;


public class ImportSedimentYield {

    private static Logger log = Logger.getLogger(ImportSedimentYield.class);

    private ImportGrainFraction grainFraction;

    private ImportUnit unit;

    private ImportTimeInterval timeInterval;

    private String description;

    private List<ImportSedimentYieldValue> values;

    private SedimentYield peer;

    public ImportSedimentYield(String description) {
        this.values = new ArrayList<ImportSedimentYieldValue>();
        this.description = description;
    }

    public void setTimeInterval(ImportTimeInterval timeInterval) {
        this.timeInterval = timeInterval;
    }

    public void setUnit(ImportUnit unit) {
        this.unit = unit;
    }

    public void setGrainFraction(ImportGrainFraction grainFraction) {
        this.grainFraction = grainFraction;
    }

    public void addValue(ImportSedimentYieldValue value) {
        this.values.add(value);
    }

    public void storeDependencies(River river) {
        log.debug("store dependencies");

        if (grainFraction != null) {
            grainFraction.storeDependencies();
        }

        SedimentYield peer = getPeer(river);

        if (peer != null) {
            int i = 0;

            for (ImportSedimentYieldValue value : values) {
                value.storeDependencies(peer);
                i++;
            }

            log.info("stored " + i + " sediment yield values.");
        }
    }

    public SedimentYield getPeer(River river) {
        log.debug("get peer");

        GrainFraction gf = grainFraction != null ? grainFraction.getPeer()
            : null;

        Unit u = unit != null ? unit.getPeer() : null;

        TimeInterval ti = timeInterval != null ? timeInterval.getPeer() : null;

        if (ti == null || u == null) {
            log.warn("Skip invalid SedimentYield: time interval or unit null!");
            return null;
        }

        if (peer == null) {
            Session session = ImporterSession.getInstance()
                .getDatabaseSession();
            Query query = session.createQuery("from SedimentYield where "
                + "   river=:river and "
                + "   grainFraction=:grainFraction and " + "   unit=:unit and "
                + "   timeInterval=:timeInterval and "
                + "   description=:description");

            query.setParameter("river", river);
            query.setParameter("grainFraction", gf);
            query.setParameter("unit", u);
            query.setParameter("timeInterval", ti);
            query.setParameter("description", description);

            List<SedimentYield> yields = query.list();
            if (yields.isEmpty()) {
                log.debug("create new SedimentYield");

                peer = new SedimentYield(river, u, ti, gf, description);
                session.save(peer);
            }
            else {
                peer = yields.get(0);
            }
        }

        return peer;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
